<?php
/**
 *  @package Virtualmoney
 *  @copyright Copyright (c)2013 JoomPROD
 *  @license GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

include_once('cparams.php');

class VirtualmoneyHelperFormat
{	
    /**
     * Return the price formatted with the proper currency config
     * 
     * @staticvar type $currencies
     * @param float $price
     * @return string
     */
    public static function formatPrice($price, $addcurrencycode = true) 
	{
        $currency = self::getConfigFormatCurrency();
        
        $number = number_format($price,$currency->numberDecimals,$currency->decimalSeparator,$currency->thousandsSeparator);
        
        if($currency->codePlural != '') {
            if($number > 1) {
                $currencyLabel = $currency->codePlural;
            } else {
                $currencyLabel = $currency->code;
            }
        } else {
            $currencyLabel = $currency->code;
        }
        if($addcurrencycode){
            if ($currency->position == "after") {
                return $number."&nbsp;".$currencyLabel;
            } else {
                return $currencyLabel."&nbsp;".$number;
            }
        }else{
            return $number;
        }
        
		return "";
	}
    
    /**
     * Return the currency config
     * 
     * @return Object $currency
     */
	public static function getConfigFormatCurrency()
	{
        $currency = new JObject();
        $currency->symbol = VirtualmoneyHelperCparams::getParam('currency_symbol','P');
        $currency->code = VirtualmoneyHelperCparams::getParam('currency_code','credit');
        $currency->codePlural = VirtualmoneyHelperCparams::getParam('currency_code_plural','credits');
        $currency->position = VirtualmoneyHelperCparams::getParam('currency_symbol_position','');
        $currency->decimalSeparator = VirtualmoneyHelperCparams::getParam('currency_decimal_separator','');
        $currency->thousandsSeparator = VirtualmoneyHelperCparams::getParam('currency_thousands_separator','');
        $currency->numberDecimals = VirtualmoneyHelperCparams::getParam('currency_number_decimal',2);
        
		return $currency;
	}
    
    /**
     * Return the substring of date before the separator in a string
     * 
     * @return string $format
     */
	public static function getConfigFormatDate()
	{
		$format =  VirtualmoneyHelperCparams::getParam('date_format','%Y-%m-%d');

		return $format;
	}
	
    /**
     * Return the string $date formatted by the configuration data format date
     * 
     * @param string $date
     * @return string
     */
	public static function formattedDate($date) {
		
		$format = self::getConfigFormatDate();
		return (strftime($format, strtotime($date)));
	}
    
    /**
     * Replaces tags in the $body by the corresponding values
     * 
     * @staticvar array $substitutions
     * @param string $body
     * @param object $user1
     * @param object $user2
     * @return string
     */
    public static function replaceTags($body,$user1,$user2) {
		static $substitutions;
        
        $substitutions = array(
                "{user1}" => $user1->username,
                "{user2}" => $user1->username);
		
		$regex_item    = '/{item_start}(.*){item_end}/s';
	
		if (preg_match( $regex_item, $body, $matches)){
			$itemHTML =  $matches[1];
			$textToReplace = $matches[0];
	
	
			$newItemHTML = "";
			$body =  str_replace($textToReplace,$newItemHTML,$body);
		}
		
		$content = str_replace(array_keys($substitution),array_values($substitution), $body);
		
		$content = self::replaceBrackets($content);
		
		return $content;
	}
	
}