<?php
/**
 *  @package Invoicing
 *  @copyright Copyright (c)2012 JoomPROD
 *  @license GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

include_once(JPATH_ADMINISTRATOR.'/components/com_invoicing/helpers/cparams.php');
include_once(JPATH_ADMINISTRATOR.'/components/com_invoicing/helpers/format.php');
include_once(JPATH_ADMINISTRATOR.'/components/com_invoicing/helpers/pdf.php');
include_once(JPATH_ADMINISTRATOR.'/components/com_invoicing/assets/view.pdf.php');
jimport( 'joomla.filesystem.file' );


	define("INVOICE_CANCELLED", "undefined");
	define("INVOICE_PAID", 3);
	define("INVOICE_NEW", 1);
	define("INVOICE_PENDING", 4);
	define("INVOICE_PAID_ADMIN", 6);
	define("INVOICE_NEW_ADMIN", 5);
	define("INVOICE_PENDING_ADMIN", 7);
	
class InvoicingHelperMail {
   
    /**
     * Mail function (uses phpMailer)
     *
     * @param   string   $from         From email address
     * @param   string   $fromname     From name
     * @param   mixed    $recipient    Recipient email address(es)
     * @param   string   $subject      Email subject
     * @param   string   $body         Message body
     * @param   boolean  $mode         False = plain text, true = HTML
     * @param   mixed    $cc           CC email address(es)
     * @param   mixed    $bcc          BCC email address(es)
     * @param   mixed    $attachment   Attachment file name(s)
     * @param   mixed    $replyto      Reply to email address(es)
     * @param   mixed    $replytoname  Reply to name(s)
     *
     * @return  boolean  True on success
     *
     * @see     JMail::sendMail()
     * @since   11.1
     */
    protected static function sendMail($from, $fromname, $recipient, $subject, $body, $mode = 0, $cc = null, $bcc = null, $attachment = null,
                $replyto = null, $replytoname = null)
    {	
		$id = JRequest::getVar('id','');
		
		$invoice = FOFModel::getTmpInstance('Invoices','InvoicingModel')
		->savestate(0)
		->getItem($id);
		
		$body = InvoicingHelperFormat::replaceTags($body,$invoice);
		$subject = InvoicingHelperFormat::replaceTags($subject,$invoice);
	
        if (version_compare(JVERSION,'2.5.0','>=')) {
            // Get a JMail instance
			$mail = JFactory::getMailer();
            $send = $mail->sendMail($from, $fromname, $recipient, $subject, $body, $mode, $cc, $bcc, $attachment,$replyto,$replytoname);

        } else {
            $send = JUtility::sendMail($from, $fromname, $recipient, $subject, $body, $mode, $cc, $bcc, $attachment,$replyto,$replytoname);
        }
		$type = gettype($send);
		if(is_object($send)) {
			$send = $send->getMessage() ;
		}
		return $send;
    }
    
    public static function getPDFInvoice($invoice) {
		$filename =  JPATH_SITE.'/tmp/invoice'.$invoice->invoicing_invoice_id.'.pdf';
		$data = InvoicingHelperFormat::formatInvoiceHTML($invoice);
		InvoicingHelperPDF::generate($data,$filename);
		return $filename;
	}

	public static function sendMailByStatus($invoice) {
		$body = "undefined";
		$status = $invoice->status;
		
		switch($status) {
			case 'PAID': $emailid = INVOICE_PAID;break;
		    case 'NEW': $emailid = INVOICE_NEW ;break;
			case 'PENDING': $emailid = INVOICE_PENDING;break;
			default: $emailid = null;
		}	
		
		if ($emailid == null)
			return;
		
		$email = FOFModel::getTmpInstance('Emails','InvoicingModel')
		->savestate(0)
		->getItem($emailid);
		
		$from = InvoicingHelperCparams::getParam('sender_mail','');
		$fromname = InvoicingHelperCparams::getParam('sender_name','');
		
		if(!isset($from) || $from == '' ){
			if (version_compare(JVERSION,'3.0.0','>=')) {
				$versionJoomla = 1;
			} else {
				$versionJoomla = 0;
			}
			$config	= JFactory::getConfig();
			$from = $versionJoomla ? $config->get('mailfrom') : $config->getValue('config.mailfrom');
			$fromname = $versionJoomla ? $config->get('fromname') : $config->getValue('config.fromname');
		}
					
		if ($from == "") {
			echo "You need to set email in invoicing configuration";
			return;
		}
		
		$email->content = InvoicingHelperFormat::replaceTags($email->content,$invoice);
		$email->subject = InvoicingHelperFormat::replaceTags($email->subject,$invoice);
		
		$sendToAdmin = InvoicingHelperCparams::getParam('maillinglist',1);
		
		//$attachment = JRequest::getInt('PDFAttachment', 'bla');
		$attachment = $email->pdf;
		
		if ($sendToAdmin) {
			//$bcc = array($from);
			
			switch($status) {
				case 'PAID': $emailidAdmin = INVOICE_PAID_ADMIN;break;
				case 'NEW': $emailidAdmin = INVOICE_NEW_ADMIN ;break;
				case 'PENDING': $emailidAdmin = INVOICE_PENDING_ADMIN;break;
				default: $emailidAdmin = null;
			}	
			
			$emailAdmin = FOFModel::getTmpInstance('Emails','InvoicingModel')
			->savestate(0)
			->getItem($emailidAdmin);
			
			$attachmentAdmin = $emailAdmin->pdf;
			
			$emailAdmin->content = InvoicingHelperFormat::replaceTags($emailAdmin->content,$invoice);
			$emailAdmin->subject = InvoicingHelperFormat::replaceTags($emailAdmin->subject,$invoice);
		}
		
		$result = "";
		
		if ($attachment)
			$attachment = self::getPDFInvoice($invoice);
		if ($attachmentAdmin)
			$attachmentAdmin = $attachment;

		if (($email->enabled)&&(isset($invoice->buyer->email))&&($invoice->buyer->email != null)) {
			 $resultUser = self::sendMail($from, $fromname,$invoice->buyer->email, $email->subject, $email->content, 1, null, null, $attachment);
		}		
		
		if ($emailAdmin->enabled && $sendToAdmin) {
		 	$resultAdmin = self::sendMail($from, $fromname, $from, $emailAdmin->subject, $emailAdmin->content, 1, null,null, $attachmentAdmin); 
		}
		
			
		return true;
	}
	
	/**
	 * Send different types of mails depends on the status of the invoice
	 *
	 */
	 
	public static function sendPaymentConfirmationEmail() {
		self::sendMailByStatus(JText::_('INVOICING_INVOICE_PAID'));
	}
	
	public static function sendInvoiceConfirmationEmail() {
		self::sendMailByStatus(JText::_('INVOICING_INVOICE_NEW'));
	}
	
	public static function sendPaymentReminder() {
		self::sendMailByStatus(JText::_('INVOICING_INVOICE_PENDING'));
	}
}
