<?php
/**
 * @package Invoicing
 * @copyright Copyright (c)203 Juloa.com
 * @license GNU General Public License version 3, or later
 */

// Protect from unauthorized access
defined('_JEXEC') or die();

include_once(JPATH_ADMINISTRATOR.'/components/com_invoicing/helpers/cparams.php');
include_once(JPATH_ADMINISTRATOR.'/components/com_invoicing/helpers/mail.php');

class InvoicingModelInvoices extends FOFModel
{
    public function qn($name) {
        $db = $this->getDbo();        
       if(version_compare(JVERSION, '3.0', 'ge')) {
			return $db->qn($name);
		} elseif(version_compare(JVERSION, '1.6', 'ge')) {
			return  $db->quoteName($name);
		}else {
            return $db->nameQuote($name);
		}           
    }
    
    public function q($name) {
        $db = $this->getDbo();        
        if(version_compare(JVERSION, '1.6', 'ge')) {
			return  $db->q($name);
		}else {
            return $db->Quote($name);
		}           
    }
    
	public function buildQuery($overrideLimits = false) {
		$db = $this->getDbo();

		$query = FOFQueryAbstract::getNew($db)
			->select('*,EXTRACT(YEAR FROM created_on) as year')
			->from($this->qn('#__invoicing_invoices'));
		
		$status = $this->getState('status','','cmd');
		
		$vendor_id = $this->getState('vendor_id','','cmd');		
		$user_id = $this->getState('user_id','','cmd');	
		
		if ($vendor_id != '') {
		$query->where($this->qn('vendor_id').' = '.$vendor_id);
		}
		
		if ($user_id != '') {
		$query->where($this->qn('user_id').' = '.$user_id);
		}
		
		if ($status != '') {
			$formattedStatus = "'".$status."'";
			$query->where($this->qn('status').' LIKE '.$formattedStatus);
		}

		$coupon_id = $this->getState('coupon_id','','cmd');
		
		if ($coupon_id != '') {
			$query->where($this->qn('coupon_id').' = '.$coupon_id);
		}
		
		$processor = $this->getState('processor','','cmd');
		
		if ($processor != '') {
			$query->where($this->qn('processor').' = '.$this->q($processor));
		}
		
 		$invoice = $this->getState('invoicing_invoice_id','','cmd');
		if(!empty($invoice)) {
			$query->where($this->qn('invoicing_invoice_id').' LIKE '.$this->q($invoice));
		} 
		
		$search = $this->getState('search',null);
		$dateFrom = $this->getState('dateFilterFrom');
		$dateTo = $this->getState('dateFilterTo');
			
		$queryVendors = FOFQueryAbstract::getNew($db)
			->select('invoicing_vendor_id')
			->from($this->qn('#__invoicing_vendors'));
			
		$cb = InvoicingHelperCparams::getParam('cb',0);
        $sql_search = '%'.$search.'%';
		if ($cb == 1) {
			$cb_name = InvoicingHelperCparams::getParam('cb_name','name');
                        if ($cb_name == "name") {
                                $cbname = "u.".$cb_name;
                                $cb_name = "u.".$cb_name." as cb_businessname";
                        } else {
                                $cbname = "c.".$cb_name;
                                $cb_name = "c.".$cb_name." as cb_businessname";
                        }
			$queryUsers = "SELECT i.invoicing_user_id FROM #__invoicing_users as i LEFT JOIN #__comprofiler as c ON c.user_id = i.user_id LEFT JOIN #__users as u ON i.user_id = u.id WHERE $cbname LIKE ".$this->q($sql_search);
		} else {
			$queryUsers = FOFQueryAbstract::getNew($db)
                        ->select('invoicing_user_id')
                        ->from($this->qn('#__invoicing_users'))
                        ->where($this->qn('businessname').' LIKE '.$this->q($sql_search).'');
		}
		
		if($search)
		{
			$query->where(
				' ('.
 				' ('.$this->qn('invoice_number').' = '.(int)$search .') OR'.
				' ('.$this->qn('order_number').' = '.(int)$search .') OR'.
				' ('.$this->qn('user_id').' = '.(int)$search .') OR'.
				' ('.$this->qn('vendor_id').' = '.(int)$search .') OR'.
				$this->qn('vendor_id').' IN ('.$queryVendors->where($this->qn('contact_name').' LIKE '.$this->q($sql_search).'').') OR'.
				$this->qn('user_id').' IN ('.$queryUsers.') OR '.
				 ' ('.$this->qn('invoicing_invoice_id').' = '.$this->q($sql_search).')'.
				')'
			);
		}
		
		if($dateFrom != "" || $dateTo != "") {
			$whereClause = "";
			
			if($dateFrom != "") {
				$whereClause .= $this->qn('created_on').' >= '.$this->q($dateFrom);	
			} 
			
			if(!empty($dateFrom) && !empty($dateTo)) {
				//echo "Les deux sont remplis";
				$whereClause .= ' AND ';
			}
			
			if($dateTo != "") {
				$whereClause .= $this->qn('created_on').' <= '.$this->q($dateTo." 24:00:00");		
			} 
			
			$query->where($whereClause);
		}
		
		//echo $query;
		
		
		$order = $this->getState('filter_order', 'invoicing_invoice_id', 'cmd');
		if(!in_array($order, array_keys($this->getTable()->getData()))) $order = 'invoicing_invoice_id';
		$dir = $this->getState('filter_order_Dir', 'desc', 'cmd');
        if ($order == "invoice_number") {
			$order = "year $dir, invoice_number";
		}
		else if ($order == "order_number") {
            $order = "year $dir,order_number";
        }
        $query->order($order.' '.$dir);
		
		return $query;
	}
	
	public function onAfterSave(&$table) {		
		//From $params
		$order_number_counter = InvoicingHelperCParams::getParam('order_number_counter',5);
		$invoice_number_counter = InvoicingHelperCParams::getParam('invoice_number_counter',5);
		
		$tmp_order_number = $table->order_number;
		$tmp_invoice_number = $table->invoice_number;

		$invoice_id =$table->invoicing_invoice_id;
		$itemids = $this->getState('itemid',array());
		if (count($itemids) > 0) {			
			$quantities = $this->getState('quantity',array());
			$descriptions = $this->getState('description',array());
			$gross_unit_prices = $this->getState('gross_unit',array());
			$net_unit_prices = $this->getState('net_unit',array());
			$taxes = $this->getState('tax',array());
			$db =JFactory::getDBO();
	
			$keepitems = array();
			foreach($itemids as $key =>$itemid) {
				if ($itemid != "") {
					$keepitems[] = $itemid;
				}
			}
			if (count($keepitems)>0) {
				$db->setQuery("DELETE FROM #__invoicing_invoice_items WHERE invoice_id = $invoice_id AND invoicing_invoice_item_id NOT IN(".implode(',',$keepitems).")");
			} else {
				$db->setQuery("DELETE FROM #__invoicing_invoice_items WHERE invoice_id = $invoice_id");
			}
			$db->query();
	
			foreach($itemids as $key =>$itemid) {
				$obj = new stdClass();
				$obj->quantity = $quantities[$key];
				$obj->description = $descriptions[$key];
				$obj->gross_unit_price = $gross_unit_prices[$key];
				$obj->net_unit_price = $net_unit_prices[$key];
				$obj->net_amount = $obj->quantity * $obj->net_unit_price;
				$obj->gross_amount = $obj->quantity * $obj->gross_unit_price;
				$obj->tax = $taxes[$key];
				$obj->ordering = $key;
				$obj->invoice_id = $invoice_id;
				if ($itemid != "") {
					$obj->invoicing_invoice_item_id = $itemid;
					$db->updateObject('#__invoicing_invoice_items', $obj,'invoicing_invoice_item_id');
				} else {
					$db->insertObject('#__invoicing_invoice_items', $obj);
				}
			}
		}
        
		if ($table->order_number == null) {
			$table->order_number = $order_number_counter;
			$order_number_counter++;
			InvoicingHelperCparams::setParam($order_number_counter,'order_number_counter');
			$table->store();
		}

		if (($table->status=="PAID")&&($table->invoice_number == null )) {
			$table->invoice_number = $invoice_number_counter;
			$invoice_number_counter++;
			InvoicingHelperCparams::setParam($invoice_number_counter,'invoice_number_counter');	
			$table->store();

            $invoice = $this->getItem();
            
			jimport('joomla.plugin.helper');
			JPluginHelper::importPlugin('invoicinggenerator');
			$app = JFactory::getApplication();
			$jResponse = $app->triggerEvent('onInvoicingPaymentValidation',array($invoice));
		} else {
            $invoice = $this->getItem();
        }
		
		$sendMailToClient = JRequest::getInt('sendMailToClient', -1);
		if ($table->status =="PENDING") {
			if (($sendMailToClient == 1) || (($sendMailToClient == -1) && ($tmp_order_number == null))) {
				$email = InvoicingHelperMail::sendMailByStatus($invoice);
			}
		} else if ($table->status =="PAID") {
			if (($sendMailToClient == 1) || (($sendMailToClient == -1) && ($tmp_invoice_number == null))) {
				$email = InvoicingHelperMail::sendMailByStatus($invoice);
			}
		}
	}
	
	public function onAfterGetItem(&$record) {
	
		$id = $record->invoicing_invoice_id;
		if ($id != null) {
			$db =JFactory::getDBO();
			$db->setQuery("SELECT * FROM #__invoicing_invoice_items WHERE invoice_id=".$id." ORDER BY ordering ASC");
			$list = $db->loadObjectList();
			$record->items = $list;
		
			$db->setQuery("SELECT * FROM #__invoicing_vendors WHERE invoicing_vendor_id=".(int)$record->vendor_id);
			$vendor = $db->loadObject();
			$record->vendor = $vendor;

			$record->buyer = FOFModel::getTmpInstance('Users','InvoicingModel')
						->savestate(0)
						->getItem($record->user_id);
			
			$db->setQuery("SELECT code as coupon,title as coupon_title FROM #__invoicing_coupons WHERE invoicing_coupon_id=".(int)$record->coupon_id);
			$c = $db->loadObject();
            if($c != null) {
                $record->coupon = $c->coupon;
                $record->coupon_title = $c->coupon_title;
            }
		} else {
			$record->number= InvoicingHelperCParams::getParam('invoice_number_counter',1);
			$record->created_on = date("Y-m-d");
			$record->coupon = "";
			$record->buyer = new stdClass();
			$record->vendor = new stdClass();
			$record->items = null;
		}
	}
	
	/**
	 * Apply Coupon to Invoice (check item types, coupon type,etc..)
	 * @param Coupon $coupon
	 * @param Invoice $invoice
	 */
	public function setCoupon($coupon,$invoice) {
		$newinvoice = new stdClass();
		$newinvoice->invoicing_invoice_id = $invoice->invoicing_invoice_id;
		$coupontypes = explode (",",$coupon->apply_on);
		
		if (in_array("0",$coupontypes)) {
			$alltypes = true;	
		} else {
			$alltypes = false;
		}
		
		$result = false;
		foreach($invoice->items as $i) 
		{
				if ($alltypes || (in_array($i->source,$coupontypes))) {
					$result = true;
				} 
		}
		
		if ($coupon->valuetype == "percent") {
			$discount_netamount = 0;
			$discount_grossamount = 0;
			
			foreach($invoice->items as $i) 
			{
				if ($alltypes || (in_array($i->source,$coupontypes))) {
					$discount_netamount += ($i->quantity * $i->net_unit_price * $coupon->value / 100);
					$discount_grossamount += ($i->quantity * $i->gross_unit_price * $coupon->value / 100);
				} 
			}
			$discount_taxamount = $discount_netamount-$discount_grossamount;
		} else {
			$discount_netamount = 0;
			$discount_grossamount = 0;
			
			$available_net_discount = $coupon->value;
			foreach($invoice->items as $i)
			{
				if ($alltypes || (in_array($i->source,$coupontypes))) {
					$itemprice = $i->quantity * $i->net_unit_price;
					if ($available_net_discount >= $itemprice) {
						$available_net_discount -= $itemprice;
						$discount_netamount += $itemprice;
						$discount_grossamount += $itemprice / ((100 + $i->tax) / 100);
					} else {
						$discount_netamount += $available_net_discount;
						$discount_grossamount += $available_net_discount / ((100 + $i->tax) / 100);
						$available_net_discount = 0;
					}
				}
				if ($available_net_discount == 0) {
					break;
				}
			}
		}

		$newinvoice->net_amount = number_format($invoice->net_subamount - $discount_netamount,2);
		$newinvoice->gross_amount = number_format($invoice->gross_subamount - $discount_grossamount,2);
		if ($newinvoice->gross_amount == -0) {
			$newinvoice->gross_amount = 0;
		}
		$newinvoice->tax_amount = number_format($newinvoice->net_amount - $newinvoice->gross_amount,2);
		
		$newinvoice->net_discount_amount = number_format($discount_netamount,2);
		$newinvoice->gross_discount_amount = number_format($discount_grossamount,2);
		$newinvoice->tax_discount_amount = number_format($discount_taxamount,2);
		
		$newinvoice->coupon_id = $coupon->invoicing_coupon_id;
		$newinvoice->discount_type = $coupon->valuetype;
		$newinvoice->discount_value = $coupon->value;
		
		$db =JFactory::getDBO();
		$db->updateObject("#__invoicing_invoices",$newinvoice,'invoicing_invoice_id');
		
		return $result;
	}
	
	function removeCoupon($invoice) 
	{
		$newinvoice = new stdClass();
		$newinvoice->invoicing_invoice_id = $invoice->invoicing_invoice_id;
		
		$newinvoice->net_amount = $invoice->net_subamount;
		$newinvoice->gross_amount = $invoice->gross_subamount;
		$newinvoice->tax_amount = $invoice->tax_subamount;
		
		$newinvoice->net_discount_amount = 0;
		$newinvoice->gross_discount_amount = 0;
		$newinvoice->tax_discount_amount = 0;
		
		$newinvoice->coupon_id = 0;
		$newinvoice->discount_type = "percent";
		$newinvoice->discount_value = 0;
	
		$db =JFactory::getDBO();
		$db->updateObject("#__invoicing_invoices",$newinvoice,'invoicing_invoice_id');
	}
	
	 function isCouponUserLimitExceeded($coupon,$user_id) {
		if ($coupon->userhitslimit == 0)
			return false;
		else {
			//If User is not login and userhits is set, cannot use the coupon
			if(JFactory::getUser()->guest) {
				return true;
			}
			$db = JFactory::getDBO();
			$query = " SELECT count(*) FROM #__invoicing_invoices "
				   . " WHERE coupon_id='".$coupon->invoicing_coupon_id
				   . "' AND user_id='".$user_id."'";
			
			$this->_db->setQuery($query);
			$nb = $this->_db->loadResult();

			if ($nb >= $coupon->userhitslimit) {
				return true;
			} else {
				return false;	
			}
		}
	}
	
	public function onBeforeSave(&$data,&$table) {
		// If invoice is going to be Paid (previous != PAID, new == PAID)
		// Update coupon hits
		if ($table->status != "PAID" && $data['status'] == "PAID") {
			$coupon_id = null;
			if ($data['coupon_id'] != null) {
				$coupon_id = $data['coupon_id'];
			} else if ($table->coupon_id) {
				$coupon_id = $table->coupon_id;
			}
			
			if ($coupon_id == null) {
				return true;
			}
			
			$coupon = FOFModel::getTmpInstance('Coupons','InvoicingModel')->getItem($coupon_id);
			
			if($coupon->invoicing_coupon_id == '') {
				return true;
			}
			
			$newcoupon = new stdClass();
			$newcoupon->invoicing_coupon_id = $coupon->invoicing_coupon_id;
			$newcoupon->hits = $coupon->hits + 1;
			$db =JFactory::getDBO();
			$db->updateObject("#__invoicing_coupons",$newcoupon,'invoicing_coupon_id');
			
		}
		
		return true;
	} 
	
	
	function getNumberOfPendingInvoices()  {
		$db = JFactory::getDBO();
		$status = "PENDING";
		$query = FOFQueryAbstract::getNew();
		$query->select('COUNT(invoicing_invoice_id)');
		$query->from($this->qn('#__invoicing_invoices'));
		$query->where( $this->qn('status') .' LIKE '."'".$status."'" );

		$db->setQuery($query);
		$count = $db->loadResult();
		
		return $count;
	}

    function getNumberAndSumBySpecifiedTime($period) {
	
		switch($period) {
			case "lastyear" : 
				$query = "SELECT COUNT(*) as number, SUM( `net_amount` ) as sum FROM #__invoicing_invoices WHERE status = 'PAID' AND YEAR(`created_on`) = YEAR(DATE_SUB(CURDATE(), INTERVAL 1 YEAR))";
				break;
			case "thisyear" : 
				$query = "SELECT COUNT(*) as number, SUM( `net_amount` ) as sum FROM #__invoicing_invoices WHERE status = 'PAID' AND YEAR(`created_on`) = YEAR(CURDATE())";
				break;
			case "lastmonth" : 
				$query = "SELECT COUNT(*) as number, SUM( `net_amount` ) as sum FROM #__invoicing_invoices WHERE status = 'PAID' AND MONTH(`created_on`) = MONTH(DATE_SUB(CURDATE(), INTERVAL 1 MONTH)) AND YEAR(`created_on`) = YEAR(CURDATE())";
				break;
			case "thismonth" : 
				$query = "SELECT COUNT(*) as number, SUM( `net_amount` ) as sum FROM #__invoicing_invoices WHERE status = 'PAID' AND MONTH(`created_on`) = MONTH(CURDATE()) AND YEAR(`created_on`) = YEAR(CURDATE())";
				break;
			case "lastsevendays" : 
				$query = "SELECT COUNT(*) as number, SUM( `net_amount` ) as sum FROM #__invoicing_invoices WHERE status = 'PAID' AND DATE(`created_on`) >= DATE_SUB(CURDATE(), INTERVAL 7 DAY)";
				break;
			case "yesterday" : 
				$query = "SELECT COUNT(*) as number, SUM( `net_amount` ) as sum FROM #__invoicing_invoices WHERE status = 'PAID' AND DATE(`created_on`) >= DATE_SUB(CURDATE(), INTERVAL 1 DAY)";
				break;
			case "today" : 
				$query = "SELECT COUNT(*) as number, SUM( `net_amount` ) as sum FROM #__invoicing_invoices WHERE status = 'PAID' AND `created_on` = CURDATE()";
				break;
			default : 
				$query = "SELECT COUNT(*) as number, SUM( `net_amount` ) as sum FROM #__invoicing_invoices WHERE status = 'PAID'";
		}
		
		$db = JFactory::getDBO();
		$db->setQuery($query);
		$list = $db->loadObjectList();
		$result = array ($list[0]->number,$list[0]->sum);
		
		return $result;
	}

	function getCAHTbetweenDaysFilters($dateFilterFrom,$dateFilterTo) {
		$db = JFactory::getDBO();
		if ($dateFilterFrom != '' &&  $dateFilterTo != '') {
			
			$query="SELECT DATE( `created_on` ) as date, SUM( `net_amount` ) as sum FROM #__invoicing_invoices WHERE status = 'PAID' AND `created_on` >= '$dateFilterFrom' AND `created_on` <= '$dateFilterTo' GROUP BY DAY( `created_on` ) , MONTH( `created_on` ) , YEAR( `created_on` )";
		}
		else {
			$query="SELECT DATE( `created_on` ) as date, SUM( `net_amount` ) as sum FROM #__invoicing_invoices WHERE status = 'PAID' AND MONTH(`created_on`) = MONTH(CURDATE()) AND YEAR(`created_on`) = YEAR(CURDATE()) GROUP BY DAY( `created_on` ) , MONTH( `created_on` ) , YEAR( `created_on` )";
			$db->setQuery($query);
		}
		$db->setQuery($query);
		$result = $db->loadObjectList();
		return $result;
	}
	
	function getCAHTbetweenMonthsFilters($monthto,$yearto,$monthfrom,$yearfrom) {
		$db = JFactory::getDBO();
		$query="SELECT DATE( `created_on` ) as date, SUM( `net_amount` ) as sum
FROM #__invoicing_invoices WHERE status = 'PAID' AND (MONTH (`created_on`) >= '$monthfrom' AND YEAR(`created_on`) = '$yearfrom')  AND (MONTH (`created_on`) <= '$monthto' AND YEAR(`created_on`) = '$yearto') GROUP BY MONTH( `created_on` ) , YEAR( `created_on` )";
		$db->setQuery($query);
		$result = $db->loadObjectList();
		
		return $result;
	}
}
