<?php
/**
 * @package Invoicing
 * @copyright Copyright (c)203 Juloa.com
 * @license GNU General Public License version 3, or later
 */

// Protect from unauthorized access
defined('_JEXEC') or die();

include_once(JPATH_ADMINISTRATOR.'/components/com_invoicing/helpers/cparams.php');
include_once(JPATH_ADMINISTRATOR.'/components/com_invoicing/helpers/mail.php');

class InvoicingModelQuotes extends FOFModel
{
    public function qn($name) {
        $db = $this->getDbo();        
       if(version_compare(JVERSION, '3.0', 'ge')) {
			return $db->qn($name);
		} elseif(version_compare(JVERSION, '1.6', 'ge')) {
			return  $db->quoteName($name);
		}else {
            return $db->nameQuote($name);
		}           
    }
    
    public function q($name) {
        $db = $this->getDbo();        
        if(version_compare(JVERSION, '1.6', 'ge')) {
			return  $db->q($name);
		}else {
            return $db->Quote($name);
		}           
    }
    
	public function buildQuery($overrideLimits = false) {
		$db = $this->getDbo();
		
		$query = FOFQueryAbstract::getNew($db)
			->select('*,EXTRACT(YEAR FROM created_on) as year')
			->from($this->qn('#__invoicing_quotes'));
		
		$status = $this->getState('status','','cmd');
		
		$vendor_id = $this->getState('vendor_id','','cmd');		
		$user_id = $this->getState('user_id','','cmd');	
		
		if ($vendor_id != '') {
		$query->where($this->qn('vendor_id').' = '.$vendor_id);
		}
		
		if ($user_id != '') {
		$query->where($this->qn('user_id').' = '.$user_id);
		}

		$coupon_id = $this->getState('coupon_id','','cmd');
		
		if ($coupon_id != '') {
			$query->where($this->qn('coupon_id').' = '.$coupon_id);
		}
		
		$processor = $this->getState('processor','','cmd');
		
		if ($processor != '') {
			$query->where($this->qn('processor').' = '.$this->q($processor));
		}
		
 		$invoice = $this->getState('invoicing_quote_id','','cmd');
		if(!empty($invoice)) {
			$query->where($this->qn('invoicing_quote_id').' LIKE '.$this->q($invoice));
		} 
		
		$search = $this->getState('search',null);
		$dateFrom = $this->getState('dateFilterFrom');
		$dateTo = $this->getState('dateFilterTo');
			
		$queryVendors = FOFQueryAbstract::getNew($db)
			->select('invoicing_vendor_id')
			->from($this->qn('#__invoicing_vendors'));
			
		$cb = InvoicingHelperCparams::getParam('cb',0);
        $sql_search = '%'.$search.'%';
		if ($cb == 1) {
			$cb_name = InvoicingHelperCparams::getParam('cb_name','name');
                        if ($cb_name == "name") {
                                $cbname = "u.".$cb_name;
                                $cb_name = "u.".$cb_name." as cb_businessname";
                        } else {
                                $cbname = "c.".$cb_name;
                                $cb_name = "c.".$cb_name." as cb_businessname";
                        }
			$queryUsers = "SELECT i.invoicing_user_id FROM #__invoicing_users as i LEFT JOIN #__comprofiler as c ON c.user_id = i.user_id LEFT JOIN #__users as u ON i.user_id = u.id WHERE $cbname LIKE ".$this->q($sql_search);
		} else {
			$queryUsers = FOFQueryAbstract::getNew($db)
                        ->select('invoicing_user_id')
                        ->from($this->qn('#__invoicing_users'))
                        ->where($this->qn('businessname').' LIKE '.$this->q($sql_search).'');
		}
		
		if($search)
		{
			$query->where(
				' ('.
				' ('.$this->qn('user_id').' = '.(int)$search .') OR'.
				' ('.$this->qn('vendor_id').' = '.(int)$search .') OR'.
				$this->qn('vendor_id').' IN ('.$queryVendors->where($this->qn('contact_name').' LIKE '.$this->q($sql_search).'').') OR'.
				$this->qn('user_id').' IN ('.$queryUsers.') OR '.
				 ' ('.$this->qn('invoicing_quote_id').' = '.$this->q($sql_search).')'.
				')'
			);
		}
		
		if($dateFrom != "" || $dateTo != "") {
			$whereClause = "";
			
			if($dateFrom != "") {
				$whereClause .= $this->qn('created_on').' >= '.$this->q($dateFrom);	
			} 
			
			if(!empty($dateFrom) && !empty($dateTo)) {
				//echo "Les deux sont remplis";
				$whereClause .= ' AND ';
			}
			
			if($dateTo != "") {
				$whereClause .= $this->qn('created_on').' <= '.$this->q($dateTo." 24:00:00");		
			} 
			
			$query->where($whereClause);
		}
		
		//echo $query;
		
		
		$order = $this->getState('filter_order', 'invoicing_quote_id', 'cmd');
		if(!in_array($order, array_keys($this->getTable()->getData()))) $order = 'invoicing_quote_id';
		$dir = $this->getState('filter_order_Dir', 'desc', 'cmd');
        $order = "year $dir, quote_number";
        $query->order($order.' '.$dir);
		
		return $query;
	}
	
	public function onAfterSave(&$table) {		
		//From $params
		$quote_number_counter = InvoicingHelperCParams::getParam('quote_number_counter',5);
		
		$tmp_quote_number = $table->quote_number;
		
		if ($table->quote_number == null) {
			$table->quote_number = $quote_number_counter;
			$quote_number_counter++;
			InvoicingHelperCparams::setParam($quote_number_counter,'quote_number_counter');
			$table->store();
		}

		$quote_id =$table->invoicing_quote_id;
		$itemids = $this->getState('itemid',array());
		if (count($itemids) > 0) {			
			$quantities = $this->getState('quantity',array());
			$descriptions = $this->getState('description',array());
			$gross_unit_prices = $this->getState('gross_unit',array());
			$net_unit_prices = $this->getState('net_unit',array());
			$taxes = $this->getState('tax',array());
			$db =JFactory::getDBO();
	
			$keepitems = array();
			foreach($itemids as $key =>$itemid) {
				if ($itemid != "") {
					$keepitems[] = $itemid;
				}
			}
			if (count($keepitems)>0) {
				$db->setQuery("DELETE FROM #__invoicing_quote_items WHERE quote_id = $quote_id AND invoicing_quote_item_id NOT IN(".implode(',',$keepitems).")");
			} else {
				$db->setQuery("DELETE FROM #__invoicing_quote_items WHERE quote_id = $quote_id");
			}
			$db->query();
	
			foreach($itemids as $key =>$itemid) {
				$obj = new stdClass();
				$obj->quantity = $quantities[$key];
				$obj->description = $descriptions[$key];
				$obj->gross_unit_price = $gross_unit_prices[$key];
				$obj->net_unit_price = $net_unit_prices[$key];
				$obj->net_amount = $obj->quantity * $obj->net_unit_price;
				$obj->gross_amount = $obj->quantity * $obj->gross_unit_price;
				$obj->tax = $taxes[$key];
				$obj->ordering = $key;
				$obj->quote_id = $quote_id;
				if ($itemid != "") {
					$obj->invoicing_quote_item_id = $itemid;
					$db->updateObject('#__invoicing_quote_items', $obj,'invoicing_quote_item_id');
				} else {
					$db->insertObject('#__invoicing_quote_items', $obj);
				}
			}
		}
		
		$quote = $this->getItem();
		
		$sendMailToClient = JRequest::getInt('sendMailToClient', -1);
        if (($sendMailToClient == 1) || (($sendMailToClient == -1) && ($tmp_quote_number == null))) {
            $email = InvoicingHelperMail::sendMailByStatus($quote);
        }
	}
	
	public function onAfterGetItem(&$record) {
	
		$id = $record->invoicing_quote_id;
		if ($id != null) {
			$db =JFactory::getDBO();
			$db->setQuery("SELECT * FROM #__invoicing_quote_items WHERE quote_id=".$id." ORDER BY ordering ASC");
			$list = $db->loadObjectList();
			$record->items = $list;
		
			$db->setQuery("SELECT * FROM #__invoicing_vendors WHERE invoicing_vendor_id=".(int)$record->vendor_id);
			$vendor = $db->loadObject();
			$record->vendor = $vendor;

			$record->buyer = FOFModel::getTmpInstance('Users','InvoicingModel')
						->savestate(0)
						->getItem($record->user_id);
			
			$db->setQuery("SELECT code as coupon,title as coupon_title FROM #__invoicing_coupons WHERE invoicing_coupon_id=".(int)$record->coupon_id);
			$c = $db->loadObject();
            if($c != null) {
                $record->coupon = $c->coupon;
                $record->coupon_title = $c->coupon_title;
            }
		} else {
			$record->number= InvoicingHelperCParams::getParam('invoice_number_counter',1);
			$record->created_on = date("Y-m-d");
			$record->coupon = "";
			$record->buyer = new stdClass();
			$record->vendor = new stdClass();
			$record->items = null;
		}
	}
	
	/**
	 * Apply Coupon to Invoice (check item types, coupon type,etc..)
	 * @param Coupon $coupon
	 * @param Invoice $invoice
	 */
	public function setCoupon($coupon,$quote) {
		$newquote = new stdClass();
		$newquote->invoicing_invoice_id = $quote->invoicing_quote_id;
		$coupontypes = explode (",",$coupon->apply_on);
		
		if (in_array("0",$coupontypes)) {
			$alltypes = true;	
		} else {
			$alltypes = false;
		}
		
		$result = false;
		foreach($quote->items as $i) 
		{
				if ($alltypes || (in_array($i->source,$coupontypes))) {
					$result = true;
				} 
		}
		
		if ($coupon->valuetype == "percent") {
			$discount_netamount = 0;
			$discount_grossamount = 0;
			
			foreach($quote->items as $i) 
			{
				if ($alltypes || (in_array($i->source,$coupontypes))) {
					$discount_netamount += ($i->quantity * $i->net_unit_price * $coupon->value / 100);
					$discount_grossamount += ($i->quantity * $i->gross_unit_price * $coupon->value / 100);
				} 
			}
			$discount_taxamount = $discount_netamount-$discount_grossamount;
		} else {
			$discount_netamount = 0;
			$discount_grossamount = 0;
			
			$available_net_discount = $coupon->value;
			foreach($quote->items as $i)
			{
				if ($alltypes || (in_array($i->source,$coupontypes))) {
					$itemprice = $i->quantity * $i->net_unit_price;
					if ($available_net_discount >= $itemprice) {
						$available_net_discount -= $itemprice;
						$discount_netamount += $itemprice;
						$discount_grossamount += $itemprice / ((100 + $i->tax) / 100);
					} else {
						$discount_netamount += $available_net_discount;
						$discount_grossamount += $available_net_discount / ((100 + $i->tax) / 100);
						$available_net_discount = 0;
					}
				}
				if ($available_net_discount == 0) {
					break;
				}
			}
		}

		$newquote->net_amount = number_format($quote->net_subamount - $discount_netamount,2);
		$newquote->gross_amount = number_format($quote->gross_subamount - $discount_grossamount,2);
		if ($newquote->gross_amount == -0) {
			$newquote->gross_amount = 0;
		}
		$newquote->tax_amount = number_format($newquote->net_amount - $newquote->gross_amount,2);
		
		$newquote->net_discount_amount = number_format($discount_netamount,2);
		$newquote->gross_discount_amount = number_format($discount_grossamount,2);
		$newquote->tax_discount_amount = number_format($discount_taxamount,2);
		
		$newquote->coupon_id = $coupon->invoicing_coupon_id;
		$newquote->discount_type = $coupon->valuetype;
		$newquote->discount_value = $coupon->value;
		
		$db =JFactory::getDBO();
		$db->updateObject("#__invoicing_quotes",$newquote,'invoicing_quote_id');
		
		return $result;
	}
	
	function removeCoupon($quote) 
	{
		$newquote = new stdClass();
		$newquote->invoicing_invoice_id = $quote->invoicing_invoice_id;
		
		$newquote->net_amount = $quote->net_subamount;
		$newquote->gross_amount = $quote->gross_subamount;
		$newquote->tax_amount = $quote->tax_subamount;
		
		$newquote->net_discount_amount = 0;
		$newquote->gross_discount_amount = 0;
		$newquote->tax_discount_amount = 0;
		
		$newquote->coupon_id = 0;
		$newquote->discount_type = "percent";
		$newquote->discount_value = 0;
	
		$db =JFactory::getDBO();
		$db->updateObject("#__invoicing_quotes",$newquote,'invoicing_quote_id');
	}
	
}
