<?php
/**
 *  @package Virtualmoney
 *  @copyright Copyright (c)2013 Joomprod
 *  @license GNU General Public License version 3, or later
 */

// Protect from unauthorized access
defined('_JEXEC') or die();

class VirtualmoneyToolbar extends FOFToolbar
{
	public function renderSubmenu()
	{
		$views = array(
				'credits',
				'emails',
				'packs',
				'orders',
				'exchanges',
				'transactions'
		);
	
		foreach($views as $label => $view) {
			if(!is_array($view)) {
				$this->addSubmenuLink($view);
			} else {
				$label = JText::_($label);
				$this->appendLink($label, '', false);
				foreach($view as $v) {
					$this->addSubmenuLink($v, $label);
				}
			}
		}
	}
	
	private function addSubmenuLink($view, $parent = null)
	{
		static $activeView = null;
		if(empty($activeView)) {
			$activeView = JRequest::getCmd('view','cpanel');
		}
	
		if ($activeView == 'cpanels')
		{
			$activeView = 'cpanel';
		}
	
		$key = strtoupper($this->component).'_TITLE_'.strtoupper($view);
		if(strtoupper(JText::_($key)) == $key) {
			$altview = FOFInflector::isPlural($view) ? FOFInflector::singularize($view) : FOFInflector::pluralize($view);
			$key2 = strtoupper($this->component).'_TITLE_'.strtoupper($altview);
			if(strtoupper(JText::_($key2)) == $key2) {
				$name = ucfirst($view);
			} else {
				$name = JText::_($key2);
			}
		} else {
			$name = JText::_($key);
		}
	
		$link = 'index.php?option='.$this->component.'&view='.$view;
	
		$active = $view == $activeView;
	
		$this->appendLink($name, $link, $active, null, $parent);
	}
	
    public function onCpanelsBrowse(){
        $option = FOFInput::getCmd('option','com_foobar',$this->input);
		JToolBarHelper::title(JText::_( strtoupper($option)), str_replace('com_', '', $option));
		JToolBarHelper::preferences('com_virtualmoney', 550, 875);
		$bar =JToolBar::getInstance('toolbar');
		$label = 'JTOOLBAR_HELP';
		$bar->appendButton( 'Popup', 'help', $label, 'http://www.joomprod.com/virtualmoney-documentation', 1000, 500 );
        $this->renderSubmenu();
    }
    
    /**
     * Set which icon will be displayed on the Transactions browser toolbar
     */
    public function onTransactionsBrowse()
	{
		// Set toolbar title
		$option = FOFInput::getCmd('option','com_foobar',$this->input);
		$subtitle_key = strtoupper($option.'_TITLE_'.FOFInput::getCmd('view','cpanel',$this->input));
		$content = JText::_( strtoupper($option)).' &ndash; <small>'.JText::_($subtitle_key).'</small>';
		$content = strip_tags(preg_replace("/&#?[a-z0-9]{2,8};/i","",$content));
		//strip tags removes <small> preg_replace removes "&ndash;"
		JToolBarHelper::title($content, str_replace('com_', '', $option));
		JToolBarHelper::preferences('com_virtualmoney', 550, 875);
		$bar =JToolBar::getInstance('toolbar');
		$label = 'JTOOLBAR_HELP';
		$bar->appendButton( 'Popup', 'help', $label, 'http://www.joomprod.com/virtualmoney-documentation', 1000, 500 );
    		
		$this->renderSubmenu();
	}
	
	public function onCreditsBrowse()
	{
		// Set toolbar title
		$option = JRequest::getCmd('option','com_foobar');
		$subtitle_key = strtoupper($option.'_TITLE_'.JRequest::getCmd('view','cpanel'));
		$content = JText::_( strtoupper($option)).' &ndash; <small>'.JText::_($subtitle_key).'</small>';
		$content = strip_tags(preg_replace("/&#?[a-z0-9]{2,8};/i","",$content));
		//strip tags removes <small> preg_replace removes "&ndash;"
		JToolBarHelper::title($content, str_replace('com_', '', $option));
	
		// Add toolbar buttons
		if($this->perms->create) {
			if(version_compare(JVERSION, '3.0', 'ge')) {
				JToolBarHelper::addNew();
			} else {
				JToolBarHelper::addNewX();
			}
		}
		if($this->perms->edit) {
			if(version_compare(JVERSION, '3.0', 'ge')) {
				JToolBarHelper::editList();
			} else {
				JToolBarHelper::editListX();
			}
		}
		if($this->perms->create || $this->perms->edit) {
			JToolBarHelper::divider();
		}
	
		if($this->perms->delete) {
			$msg = JText::_(JRequest::getCmd('option','com_foobar').'_CONFIRM_DELETE');
			JToolBarHelper::deleteList($msg);
		}
		JToolBarHelper::preferences('com_virtualmoney', 550, 875);
		$bar =JToolBar::getInstance('toolbar');
		$label = 'JTOOLBAR_HELP';
		$bar->appendButton( 'Popup', 'help', $label, 'http://www.joomprod.com/virtualmoney-documentation', 1000, 500 );
		$this->renderSubmenu();
	}
    
    /**
     * Set which icon will be displayed on the Exchanges browser toolbar
     */
    public function onExchangesBrowse()
	{
		// Set toolbar title
		$option = FOFInput::getCmd('option','com_foobar',$this->input);
		$subtitle_key = strtoupper($option.'_TITLE_'.FOFInput::getCmd('view','cpanel',$this->input));
		$content = JText::_( strtoupper($option)).' &ndash; <small>'.JText::_($subtitle_key).'</small>';
		$content = strip_tags(preg_replace("/&#?[a-z0-9]{2,8};/i","",$content));
		//strip tags removes <small> preg_replace removes "&ndash;"
		JToolBarHelper::title($content, str_replace('com_', '', $option));
		JToolBarHelper::preferences('com_virtualmoney', 550, 875);
		$bar =JToolBar::getInstance('toolbar');
		$label = 'JTOOLBAR_HELP';
		$bar->appendButton( 'Popup', 'help', $label, 'http://www.joomprod.com/virtualmoney-documentation', 1000, 500 );
		$this->renderSubmenu();
	}
    
    /**
     * Set which icon will be displayed on the Orders browser toolbar
     */
    public function onOrdersBrowse()
	{
		// Set toolbar title
		$option = FOFInput::getCmd('option','com_foobar',$this->input);
		$subtitle_key = strtoupper($option.'_TITLE_'.FOFInput::getCmd('view','cpanel',$this->input));
		$content = JText::_( strtoupper($option)).' &ndash; <small>'.JText::_($subtitle_key).'</small>';
		$content = strip_tags(preg_replace("/&#?[a-z0-9]{2,8};/i","",$content));
		//strip tags removes <small> preg_replace removes "&ndash;"
		JToolBarHelper::title($content, str_replace('com_', '', $option));
		JToolBarHelper::preferences('com_virtualmoney', 550, 875);
		$bar =JToolBar::getInstance('toolbar');
		$label = 'JTOOLBAR_HELP';
		$bar->appendButton( 'Popup', 'help', $label, 'http://www.joomprod.com/virtualmoney-documentation', 1000, 500 );
		$this->renderSubmenu();
	}
	public function onPacksBrowse()
	{
		// Set toolbar title
		$option = JRequest::getCmd('option','com_invoicing');
		$subtitle_key = strtoupper($option.'_TITLE_'.JRequest::getCmd('view','cpanel'));
		$content = JText::_( strtoupper($option)).' &ndash; <small>'.JText::_($subtitle_key).'</small>';
		$content = strip_tags(preg_replace("/&#?[a-z0-9]{2,8};/i","",$content));
		//strip tags removes <small> preg_replace removes "&ndash;"
		JToolBarHelper::title($content, str_replace('com_', '', $option));

		
		// Add toolbar buttons
		if($this->perms->create) {
			if(version_compare(JVERSION, '3.0', 'ge')) {
				JToolBarHelper::addNew();
			} else {
				JToolBarHelper::addNewX();
			}
		}
		if($this->perms->edit) {
			if(version_compare(JVERSION, '3.0', 'ge')) {
				JToolBarHelper::editList();
			} else {
				JToolBarHelper::editListX();
			}
		}
		if($this->perms->create || $this->perms->edit) {
			JToolBarHelper::divider();
		}

		if($this->perms->delete) {
			$msg = JText::_(JRequest::getCmd('option','com_invoicing').'_CONFIRM_DELETE');
			JToolBarHelper::deleteList($msg);
		}
		JToolBarHelper::preferences('com_invoicing', 550, 875);
		$bar = JToolBar::getInstance('toolbar');
		$label = 'JTOOLBAR_HELP';
		$bar->appendButton( 'Popup', 'help', $label, 'http://www.joomprod.com/virtualmoney-documentation', 1000, 500 );
		$this->renderSubmenu();
	}
	
	public function onEmailsBrowse()
	{
		// Set toolbar title
		$option = JRequest::getCmd('option','com_invoicing');
		$subtitle_key = strtoupper($option.'_TITLE_'.JRequest::getCmd('view','cpanel'));
		$content = JText::_( strtoupper($option)).' &ndash; <small>'.JText::_($subtitle_key).'</small>';
		$content = strip_tags(preg_replace("/&#?[a-z0-9]{2,8};/i","",$content));
		//strip tags removes <small> preg_replace removes "&ndash;"
		JToolBarHelper::title($content, str_replace('com_', '', $option));

		
		// Add toolbar buttons
/* 		if($this->perms->create) {
			if(version_compare(JVERSION, '3.0', 'ge')) {
				JToolBarHelper::addNew();
			} else {
				JToolBarHelper::addNewX();
			}
		} */
		
		if($this->perms->edit) {
			if(version_compare(JVERSION, '3.0', 'ge')) {
				JToolBarHelper::editList();
			} else {
				JToolBarHelper::editListX();
			}
		}
		if($this->perms->create || $this->perms->edit) {
			JToolBarHelper::divider();
		}

		JToolBarHelper::preferences('com_invoicing', 550, 875);
		$bar = JToolBar::getInstance('toolbar');
		$label = 'JTOOLBAR_HELP';
		$bar->appendButton( 'Popup', 'help', $label, 'http://www.joomprod.com/virtualmoney-documentation', 1000, 500 );
		$this->renderSubmenu();
	}
	
	public function onBrowse() 
	{
		parent::onBrowse();
		JToolBarHelper::preferences('com_virtualmoney', 550, 875);
		$bar =JToolBar::getInstance('toolbar');
		$label = 'JTOOLBAR_HELP';
		$bar->appendButton( 'Popup', 'help', $label, 'http://www.joomprod.com/virtualmoney-documentation', 1000, 500 );
	}
    
	// Do not add Help in Edit because onedit call onAdd
    public function onAdd()
	{
		parent::onAdd();
        $bar =JToolBar::getInstance('toolbar');
		$label = 'JTOOLBAR_HELP';
		$bar->appendButton( 'Popup', 'help', $label, 'http://www.joomprod.com/virtualmoney-documentation', 1000, 500 );
	}
}