<?php
/**
 *  @package Virtualmoney
 *  @copyright Copyright (c)2013 JoomPROD
 *  @license GNU General Public License version 3, or later
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

include_once(JPATH_LIBRARIES.'/fof/include.php');
require_once(JPATH_ADMINISTRATOR.'/components/com_virtualmoney/helpers/format.php');
include_once(JPATH_ADMINISTRATOR.'/components/com_virtualmoney/helpers/cparams.php');
include_once(JPATH_ADMINISTRATOR.'/components/com_virtualmoney/helpers/mail.php');

// Merge the language overrides
$paths = array(JPATH_ADMINISTRATOR, JPATH_ROOT);
$jlang = JFactory::getLanguage();
$jlang->load("com_virtualmoney", $paths[0], 'en-GB', true);
$jlang->load("com_virtualmoney", $paths[0], null, true);
$jlang->load("com_virtualmoney", $paths[1], 'en-GB', true);
$jlang->load("com_virtualmoney", $paths[1], null, true);

class Virtualmoney {
	
    /**
     * Check if the user had enough credit to purchase the total
     * 
     * @param int $total
     * @param int $userId
     * @return boolean
     */
    public static function checkCredits($total, $userId) {
        $creditModel = FOFModel::getTmpInstance('Credits','VirtualmoneyModel');
        $credits = $creditModel->user_id($userId)->getItemList();
        $credits = $credits[0];
        
        if($credits->credits < $total) {
            return false;
        }
        
        return true;
    }
    
    /**
     * Add the amount of $credits to the account of the user
     * and create a transaction
     * 
     * @param int $userId
     * @param int $newCredits
     * @param string $transactionDescription
     * @param string $source
     * @return boolean
     */
	public static function addCredits($userId, $newCredits, $transactionDescription = '', $source = '') {
        $creditModel = FOFModel::getTmpInstance('Credits','VirtualmoneyModel');
        $credits = $creditModel->user_id($userId)->getItemList();
        $credits = $credits[0];
        
        $credits->user_id = $userId;
        $credits->credits += (float)$newCredits;
        $creditModel->save($credits);
        
        //Insert the transaction
        $transaction = new JObject();
        $transaction->user_id = $userId;
        $transaction->credits = $newCredits;
        $transaction->balance = $credits->credits;
        $transaction->description = $transactionDescription;
        $transaction->type = $source;
        $transaction->created_on = date('Y-m-d H:i:s');
        
        $transactionModel = FOFModel::getTmpInstance('Transactions', 'VirtualmoneyModel');
        $transactionModel->save($transaction);
        
        return true;
    }
    
	/**
     * Remove the amount of $credits to the account of the user
     * and create a transaction
     * 
     * @param int $userId
     * @param int $newCredits
     * @param string $transactionDescription
     * @param string $source
     * @return boolean
     */
    public static function removeCredits($userId, $newCredits, $transactionDescription = '', $source = '') {
        $creditModel = FOFModel::getTmpInstance('Credits','VirtualmoneyModel');
        $credits = $creditModel->user_id($userId)->getItemList();
        $credits = $credits[0];
        
        $credits->user_id = $userId;
        $credits->credits -= (float)$newCredits;
        
        //If, after the remove, the user has less credits than 0 -> error
        if($credits->credits < 0) {
            //Error
            return false;
        }
        $creditModel->save($credits);

        //Insert the transaction
        $transaction = new JObject();
        $transaction->user_id = $userId;
        $transaction->credits = -$newCredits;
        $transaction->balance = $credits->credits;
        $transaction->description = $transactionDescription;
        $transaction->type = $source;
        $transaction->created_on = date('Y-m-d H:i:s');
        
        $transactionModel = FOFModel::getTmpInstance('Transactions', 'VirtualmoneyModel');
        $transactionModel->save($transaction);
        
        return true;
    }
    
    /**
     * get the amount of $credits available in the account of the user
     * 
     * @param int $userId
     * @return int $credits
     */
    public static function getCredits($userId, $addcurrencycode = false) {
        if ($userId != 0) {
        	$creditModel = FOFModel::getTmpInstance('Credits','VirtualmoneyModel');
        	$credit = $creditModel->user_id($userId)->getItemList();
        } else {
        	$credit = null;
        }
        
        if (count($credit) > 0)
        	return VirtualmoneyHelperFormat::formatPrice($credit[0]->credits,$addcurrencycode);
        else 
        	return VirtualmoneyHelperFormat::formatPrice(0,$addcurrencycode);
    }
    
    /**
     * Returns the transactions of the user
     * 
     * @param int $userid
     * @param int $limit
     * @param int $limitStart
     * @return array
     */
    public static function getTransactions($userId, $limit = 5, $limitStart = 0) {
        $userId = (int)$userId;
        $transactions = FOFModel::getTmpInstance('Transactions','VirtualmoneyModel')
                        ->user_id($userId)
                        ->filter_order('created_on')
                        ->filter_order_Dir('DESC')
                        ->limit($limit)
                        ->limitStart($limitStart)
                        ->getItemList();
		return $transactions;
    }
    
    /**
     * Returns the orders of the user
     * 
     * @param int $userid
     * @param int $limit
     * @param int $limitStart
     * @return array
     */
    public static function getOrders($userId, $limit = 5, $limitStart = 0) {
        $userId = (int)$userId;
        $orders = FOFModel::getTmpInstance('Orders','VirtualmoneyModel')
                        ->user_id($userId)
                        ->filter_order('created_on')
                        ->filter_order_Dir('DESC')
                        ->limit($limit)
                        ->limitStart($limitStart)
                        ->getItemList();
		return $orders;
    }
	
}
